"use strict";
/*
  Copyright © 2017 Kerry Shetline, kerry@shetline.com

  MIT license: https://opensource.org/licenses/MIT

  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit
  persons to whom the Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
  Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var debug = false;
exports.abs = Math.abs;
exports.acos = Math.acos;
exports.acosh = Math.acosh;
function acos_deg(x) {
    return Math.acos(x) / Math.PI * 180.0;
}
exports.acos_deg = acos_deg;
function acot(x) {
    return Math.PI / 2.0 - Math.atan(x);
}
exports.acot = acot;
function acot2(a, b) {
    return Math.PI / 2.0 - Math.atan2(a, b);
}
exports.acot2 = acot2;
function acot_deg(x) {
    return 180.0 - Math.atan(x) / Math.PI * 180.0;
}
exports.acot_deg = acot_deg;
function acot2_deg(a, b) {
    return 180.0 - Math.atan2(a, b) / Math.PI * 180.0;
}
exports.acot2_deg = acot2_deg;
exports.asin = Math.asin;
exports.asinh = Math.asinh;
function asin_deg(x) {
    return Math.asin(x) / Math.PI * 180.0;
}
exports.asin_deg = asin_deg;
exports.atan = Math.atan;
exports.atanh = Math.atanh;
function atan_deg(x) {
    return Math.atan(x) / Math.PI * 180.0;
}
exports.atan_deg = atan_deg;
exports.atan2 = Math.atan2;
function atan2_deg(a, b) {
    return Math.atan2(a, b) / Math.PI * 180.0;
}
exports.atan2_deg = atan2_deg;
exports.ceil = Math.ceil;
exports.cos = Math.cos;
exports.cosh = Math.cosh;
function cos_deg(x) {
    return Math.cos(x / 180.0 * Math.PI);
}
exports.cos_deg = cos_deg;
/**
 * Integer division with "truncation toward zero".
 */
function div_tt0(x, y) {
    var d = x / y;
    if (d >= 0)
        return Math.floor(d);
    else
        return Math.ceil(d);
}
exports.div_tt0 = div_tt0;
/**
 * Integer division, always rounding down to the nearest integer regardless of sign of the result.
 */
function div_rd(x, y) {
    return Math.floor(x / y);
}
exports.div_rd = div_rd;
exports.exp = Math.exp;
exports.floor = Math.floor;
function interpolate(x0, x, x1, y0, y1) {
    if (x0 === x1)
        return y0;
    return y0 + (x - x0) * (y1 - y0) / (x1 - x0);
}
exports.interpolate = interpolate;
function interpolateModular(x0, x, x1, y0, y1, modulus, signedResult) {
    if (signedResult === void 0) { signedResult = false; }
    var m2 = modulus / 2;
    if (y0 < 0 || y0 >= modulus)
        y0 = mod(y0, modulus);
    while (y1 < y0 - m2)
        y1 += modulus;
    while (y1 >= y0 + m2)
        y1 -= modulus;
    var y = interpolate(x0, x, x1, y0, y1);
    if (signedResult && (y < m2 || y >= m2))
        y = mod2(y, modulus);
    else if (y < 0 || y >= modulus)
        y = mod(y, modulus);
    return y;
}
exports.interpolateModular = interpolateModular;
function interpolateOverIntRange(xx, yy, x, a, b) {
    var y = 0.0;
    for (var i = a; i <= b; ++i) {
        var c = 1.0;
        for (var j = a; j <= b; ++j) {
            if (j !== i)
                c *= (x - xx[j]) / (xx[i] - xx[j]);
        }
        y += c * yy[i];
    }
    return y;
}
function interpolateAux(xx, yy, x, maxSpan, xc) {
    var n = Math.min(xx.length, yy.length);
    var a = -1;
    var b = n - 1;
    for (var i = 0; i < n; ++i) {
        var xi = xx[i];
        if (a < 0 && xi >= xc - maxSpan)
            a = i;
        if (xi >= xc + maxSpan) {
            b = i;
            break;
        }
    }
    return interpolateOverIntRange(xx, yy, x, Math.max(a, 0), b);
}
// Note: To use maxSpan (by specifying a positive value) array data must be pre-sorted
// by ascending x value.
//
function interpolateTabular(xx, yy, x, maxSpan) {
    var n = Math.min(xx.length, yy.length);
    if (maxSpan <= 0.0)
        return interpolateOverIntRange(xx, yy, x, 0, n - 1);
    // To avoid sudden jumps in the return value of this function as we shift
    // from one range of table values to another, we'll construct two ranges
    // based on maxSpan -- one for the tabular xx value just below x, another
    // for the xx value just above x. An interpolation will be performed over
    // both ranges, then a weighted average of the two interpolations, based
    // on the relative position of x between the two bounding xx values, will
    // be returned.
    var ca = -1;
    var cb = -1;
    for (var i = 0; i < n; ++i) {
        var xi = xx[i];
        if (xi === x)
            return yy[i];
        if (xi < x)
            ca = i;
        else if (xi > x) {
            cb = i;
            break;
        }
    }
    if (ca < 0)
        return yy[0];
    else if (cb < 0)
        return yy[n - 1];
    var xa = xx[ca];
    var xb = xx[cb];
    var weight = (x - xa) / (xb - xa);
    var ya = interpolateAux(xx, yy, x, maxSpan, xa);
    var yb = interpolateAux(xx, yy, x, maxSpan, xb);
    return ya * (1.0 - weight) + yb * weight;
}
exports.interpolateTabular = interpolateTabular;
function intersects(r1, r2) {
    return (r1.x < r2.x + r2.w && r1.x + r1.w > r2.x &&
        r1.y < r2.y + r2.h && r1.y + r1.h > r2.y);
}
exports.intersects = intersects;
function irandom(lowestOrMax, highest) {
    if (arguments.length === 1)
        return 1 + Math.floor(Math.random() * lowestOrMax);
    return lowestOrMax + Math.floor(Math.random() * (highest - lowestOrMax + 1));
}
exports.irandom = irandom;
function limitNeg1to1(x, tolerance) {
    if (tolerance === void 0) { tolerance = 0.01; }
    if (x < -1 - tolerance) {
        if (debug) {
            console.debug('Value out of range: ' + x + ' < -1.0');
            console.trace();
        }
        return -1.0;
    }
    else if (x > 1 + tolerance) {
        if (debug) {
            console.debug('Value out of range: ' + x + ' > 1.0');
            console.trace();
        }
        return 1.0;
    }
    else if (x < -1.0)
        return -1.0;
    else if (x > 1.0)
        return 1.0;
    else
        return x;
}
exports.limitNeg1to1 = limitNeg1to1;
exports.HALF_PI = Math.PI / 2.0;
exports.log = Math.log;
exports.log2 = Math.log2;
exports.log10 = Math.log10;
exports.max = Math.max;
exports.min = Math.min;
/**
 * A modulo operation complementary to div_rd(), differing from % in how the relationship between
 * the signs of the arguments and the result is handled.
 */
function mod(x, y) {
    var m = x % y;
    if ((m < 0 && y > 0) || (m > 0 && y < 0)) {
        return y + m;
    }
    return m;
}
exports.mod = mod;
/**
 * For use with values such as angles, this modulo function splits the range of the modulus
 * between negative and positive numbers, [-y/2, y/2), e.g. for a y of 360, the result ranges
 * from -180 up to (but not including) 180. This function can return non-integer values.
 */
function mod2(x, y) {
    var result = x - Math.floor(x / y) * y;
    if (result >= y / 2.0)
        result -= y;
    return result;
}
exports.mod2 = mod2;
exports.PI = Math.PI;
exports.pow = Math.pow;
function random(lowestOrMax, highest) {
    switch (arguments.length) {
        case 1: return Math.random() * lowestOrMax;
        case 2: return lowestOrMax + Math.random() * (highest - lowestOrMax);
    }
    return Math.random();
}
exports.random = random;
exports.round = Math.round;
exports.sign = Math.sign;
function signZN(x) {
    if (x > 0.0)
        return 1.0;
    else
        return -1.0;
}
exports.signZN = signZN;
function signZP(x) {
    if (x < 0.0)
        return -1.0;
    else
        return 1.0;
}
exports.signZP = signZP;
exports.sin = Math.sin;
exports.sinh = Math.sinh;
function sin_deg(x) {
    return Math.sin(x / 180.0 * Math.PI);
}
exports.sin_deg = sin_deg;
exports.sqrt = Math.sqrt;
function squared(x) {
    return x * x;
}
exports.squared = squared;
exports.tan = Math.tan;
exports.tanh = Math.tanh;
function tan_deg(x) {
    return Math.tan(x / 180.0 * Math.PI);
}
exports.tan_deg = tan_deg;
function to_degree(x) {
    return x * 180.0 / Math.PI;
}
exports.to_degree = to_degree;
function to_radian(x) {
    return x * Math.PI / 180.0;
}
exports.to_radian = to_radian;
exports.TWO_PI = Math.PI * 2;
function union(r1, r2) {
    var minX = exports.min(r1.x, r2.x);
    var minY = exports.min(r1.y, r2.y);
    var maxX = exports.max(r1.x + r1.w, r2.x + r2.w);
    var maxY = exports.max(r1.y + r1.h, r2.y + r2.h);
    return { x: minX, y: minY, w: maxX - minX, h: maxY - minY };
}
exports.union = union;
//# sourceMappingURL=ks-math.js.map