"use strict";
/*
  Copyright © 2017 Kerry Shetline, kerry@shetline.com

  MIT license: https://opensource.org/licenses/MIT

  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit
  persons to whom the Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
  Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var spherical_position_1 = require("./spherical-position");
var angle_1 = require("./angle");
var SphericalPosition3D = /** @class */ (function (_super) {
    __extends(SphericalPosition3D, _super);
    function SphericalPosition3D(longitude, latitude, _radius, longUnit, latUnit) {
        if (_radius === void 0) { _radius = 0; }
        var _this = _super.call(this, longitude, latitude, longUnit, latUnit) || this;
        _this._radius = _radius;
        return _this;
    }
    SphericalPosition3D.convertRectangular = function (xOrPoint, y, z) {
        var x;
        if (typeof xOrPoint === 'number') {
            x = xOrPoint;
            if (y === undefined || z === undefined)
                throw ('Invalid arguments');
        }
        else {
            x = xOrPoint.x;
            y = xOrPoint.y;
            z = xOrPoint.z;
        }
        return new SphericalPosition3D(angle_1.Angle.atan2_nonneg(y, x), angle_1.Angle.atan2(z, Math.sqrt(x * x + y * y)), Math.sqrt(x * x + y * y + z * z));
    };
    SphericalPosition3D.from2D = function (pos, radius) {
        return new SphericalPosition3D(pos.longitude, pos.latitude, radius);
    };
    Object.defineProperty(SphericalPosition3D.prototype, "radius", {
        get: function () {
            return this._radius;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SphericalPosition3D.prototype, "xyz", {
        get: function () {
            return {
                x: this._radius * this._latitude.cos * this._longitude.cos,
                y: this._radius * this._latitude.cos * this._longitude.sin,
                z: this._radius * this._latitude.sin
            };
        },
        enumerable: true,
        configurable: true
    });
    SphericalPosition3D.prototype.translate = function (newOrigin) {
        var L0 = newOrigin.longitude;
        var B0 = newOrigin.latitude;
        var R0 = newOrigin.radius;
        var L = this.longitude;
        var B = this.latitude;
        var R = this.radius;
        var x = R * B.cos * L.cos - R0 * B0.cos * L0.cos;
        var y = R * B.cos * L.sin - R0 * B0.cos * L0.sin;
        var z = R * B.sin - R0 * B0.sin;
        return SphericalPosition3D.convertRectangular(x, y, z);
    };
    SphericalPosition3D.prototype.toString = function () {
        return _super.prototype.toString.call(this) + ', rad: ' + this.radius.toFixed(5);
    };
    return SphericalPosition3D;
}(spherical_position_1.SphericalPosition));
exports.SphericalPosition3D = SphericalPosition3D;
//# sourceMappingURL=spherical-position-3d.js.map