"use strict";
/*
  Copyright © 2017 Kerry Shetline, kerry@shetline.com

  MIT license: https://opensource.org/licenses/MIT

  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit
  persons to whom the Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
  Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var angle_1 = require("./angle");
var ks_math_1 = require("./ks-math");
var SphericalPosition = /** @class */ (function () {
    function SphericalPosition(longitude, latitude, longUnit, latUnit) {
        if (longitude === void 0) { longitude = 0; }
        if (latitude === void 0) { latitude = 0; }
        if (longUnit === void 0) { longUnit = angle_1.Unit.RADIANS; }
        if (latUnit === void 0) { latUnit = angle_1.Unit.RADIANS; }
        if (typeof longitude === 'number')
            this._longitude = new angle_1.Angle(longitude, longUnit, angle_1.Mode.RANGE_LIMIT_NONNEGATIVE);
        else
            this._longitude = longitude;
        if (typeof latitude === 'number')
            this._latitude = new angle_1.Angle(latitude, latUnit);
        else
            this._latitude = latitude;
    }
    Object.defineProperty(SphericalPosition.prototype, "longitude", {
        get: function () {
            return this._longitude;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SphericalPosition.prototype, "rightAscension", {
        get: function () {
            return this._longitude;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SphericalPosition.prototype, "altitude", {
        get: function () {
            return this._latitude;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SphericalPosition.prototype, "azimuth", {
        get: function () {
            return this._longitude;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SphericalPosition.prototype, "latitude", {
        get: function () {
            return this._latitude;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SphericalPosition.prototype, "declination", {
        get: function () {
            return this._latitude;
        },
        enumerable: true,
        configurable: true
    });
    SphericalPosition.prototype.distanceFrom = function (p) {
        // Tiny rounding errors which can make the argument of acos very slightly larger than 1.0
        // or very slight smaller than -1.0 are enough to blow up the acos function and get you a
        // NaN for your trouble.
        //
        var d = ks_math_1.acos(ks_math_1.limitNeg1to1(this._latitude.sin * p._latitude.sin +
            this._latitude.cos * p._latitude.cos *
                this._longitude.subtract(p._longitude).cos));
        d = ks_math_1.abs(ks_math_1.mod2(d, ks_math_1.TWO_PI));
        return new angle_1.Angle(d);
    };
    SphericalPosition.prototype.toString = function () {
        return 'lon: ' + this.longitude + ', lat: ' + this.latitude;
    };
    return SphericalPosition;
}());
exports.SphericalPosition = SphericalPosition;
//# sourceMappingURL=spherical-position.js.map